#include "bone.h"
#include "skeleton.h"

void cleanup_bones(unsigned short num_bones, bone_t *bones, unsigned short num_frames) {
  if (! num_bones || ! bones) return;
  for (unsigned short i = 0; i < num_bones; i++) {
    bone_t &bone = bones[i];
    if (bone.name_length && bone.name) delete[] bone.name;
    bone.name_length = 0; bone.name = 0;
    if (! num_frames) continue;
    if (bone.orientations) delete[] bone.orientations;
    bone.orientations = 0;
    if (bone.locals) delete[] bone.locals;
    bone.locals = 0;
    if (bone.worlds) delete[] bone.worlds;
    bone.worlds = 0;
  }
  delete[] bones;
}

int read_bones(FILE *file, unsigned short num_bones, bone_t *bones) {
  unsigned short i;

  /* read bones */
  for (i = 0; i < num_bones; i++) {
    bone_t &bone = bones[i];
    if (skip(file, 2)) return 6;
    if (read_uint16(file, &bone.name_length)) return 7;
    bone.name = new char[bone.name_length + 1];
    bone.name[bone.name_length] = '\0';
    if (read_arbitrary(file, bone.name, bone.name_length)) return 8;
    if (read_uint16(file, &bone.parent)) return 9;
    if (read_vec3(file, &bone.position)) return 10;
    if (read_vec4(file, &bone.orientation)) return 11;
  }

  /* now for the calculations */
  for (i = 0; i < num_bones; i++) {
    bone_t &bone = bones[i];
    /* convert rotation to matrix */
    sgQuatToMatrix(bone.local, bone.orientation);
    /* add translation part */
    sgCopyVec3(bone.local[3], bone.position);
    /* multiply by parent's world (for root bone this is defined as I) */
    if (bone.parent == 0xffff) sgCopyMat4(bone.world, bone.local);
    else sgMultMat4(bone.world, bones[bone.parent].world, bone.local);
  }

  return 0;
}

int write_bones(FILE *file, unsigned short num_bones, bone_t *bones) {
  unsigned short i;

  /* write bones */
  for (i = 0; i < num_bones; i++) {
    bone_t &bone = bones[i];
    if (write_uint16(file, &i)) return 6;
    if (write_uint16(file, &bone.name_length)) return 7;
    if (write_arbitrary(file, bone.name, bone.name_length)) return 8;
    if (write_uint16(file, &bone.parent)) return 9;
    if (write_vec3(file, &bone.position)) return 10;
    if (write_vec4(file, &bone.orientation)) return 11;
  }

  return 0;
}

/* Make default bone */
static int make_default_bone(bone_t &bone, unsigned char *default_bone) {
  memmove(&bone.parent, default_bone, 2);
  memmove(&bone.position, (char *) default_bone + 2, 12);
  memmove(&bone.orientation, (char *) default_bone + 14, 16);

  sgQuatToMatrix(bone.local, bone.orientation);
  sgCopyVec3(bone.local[3], bone.position);

  bone.animated = 0;
  bone.orientations = 0;
  bone.locals = 0;
  bone.worlds = 0;

  return 0;
}

/* Make default SWT bone */
int make_swt_bone(bone_t &bone, unsigned short bone_id) {
  if (bone_id >= SWT_NUM_BONES) return 1;
  return make_default_bone(bone, swt_bone_coords[bone_id]);
}

/* Make default KID bone */
int make_kid_bone(bone_t &bone, unsigned short bone_id) {
  if (bone_id >= KID_NUM_BONES) return 1;
  return make_default_bone(bone, kid_bone_coords[bone_id]);
}

/* Make default TAC bone */
int make_tac_bone(bone_t &bone, unsigned short bone_id) {
  if (bone_id >= TAC_NUM_BONES) return 1;
  return make_default_bone(bone, tac_bone_coords[bone_id]);
}

/* Make default BF1 bone */
int make_bf1_bone(bone_t &bone, unsigned short bone_id) {
  if (bone_id >= BF1_NUM_BONES) return 1;
  return make_default_bone(bone, bf1_bone_coords[bone_id]);
}
