#include "main.h"

static SDL_Surface *screen;

int main(int argc, char **argv) {
  SDL_Event event;
  SDL_Joystick *joystick;
  int i;

  /* Args */
  printf("Processing command line: ");
  process_args(argc, argv);
  printf("OK\n");

  /* SDL */
  printf("SDL initialisation: ");
  SDL_VERSION(&wm.version);
  if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_JOYSTICK | SDL_INIT_TIMER)) {
    printf("failed!\n");
    return 1;
  }
  printf("version %u.%u.%u!\n", wm.version.major, wm.version.minor, wm.version.patch);

  /* screen */
  SDL_WM_SetCaption("GSkinViewer", 0);
  SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 16);
  SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
  SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_INTERVAL, SDL_DEFAULT_REPEAT_INTERVAL);
  screen = resize_window(width, height);
  if (! screen) return 2;

  /* DevIL */
  printf("Initialising DevIL: ");
  setup_devil();
  printf("OK\n");

  /* GUI initialisation (such as it is) */
  printf("Initialising GUI: ");
  setup_gui();
  printf("OK!\n");

  /* joystick */
  printf("Checking joysticks: ");
  if (joystick_enabled) {
    int num_joysticks = SDL_NumJoysticks();
    printf("%d\n", num_joysticks);
    if (num_joysticks) {
      SDL_JoystickEventState(SDL_ENABLE);
      for (i = 0; i < num_joysticks; i++) {
        printf("  Joystick 0: %s ", SDL_JoystickName(i));
        joystick = SDL_JoystickOpen(i);
        if (joystick) printf("(opened)\n");
        else printf("(cannot open!)\n");
        SDL_JoystickClose(joystick);
      } 
    }
  }
  else printf("disabled\n");

  xpos = 0.0f; ypos = -40.0f; zpos = -96.0f;
  yrot = 180.0f;
  show_bones = 0;

  exitasap = 0;
#ifdef _MSC_VER
  SDL_EventState(SDL_SYSWMEVENT, SDL_IGNORE);
#endif
  SDL_SetTimer(100, advance_frame);
  while (! exitasap) {
    if (SDL_PollEvent(&event)) exitasap = handle_event(&event);
    if (visible) {
      if (joystick_enabled) poll_joysticks();
      draw_scene();
    }
  }

  /* close surfaces */
  printf("Freeing surface: ");
  SDL_FreeSurface(screen);
  printf("OK!\n");

  printf("GUI shutdown: ");
  cleanup_gui();
  printf("OK!\n");
  
  printf("SDL shutdown: ");
  SDL_Quit();
  printf("OK!\n");

  return 0;
}

int handle_event(SDL_Event *event) {
  int ret = 0;

  switch (event->type) {
#if 0
    case SDL_MOUSEBUTTONDOWN:
      handle_button_press(event->button.button, event->button.x, event->button.y);
      break;
    case SDL_MOUSEBUTTONUP:
      handle_button_release(event->button.button, event->button.x, event->button.y);
      break;
#endif
    case SDL_MOUSEMOTION:
      poll_mouse(event->motion.state, event->motion.xrel, event->motion.yrel);
      break;
    case SDL_KEYDOWN:
      ret |= handle_key_press(&event->key);
      break;
    case SDL_KEYUP:
      ret |= handle_key_release(&event->key);
      break;
    case SDL_QUIT:
      ret |= 1;
      break;
    case SDL_SYSWMEVENT:
      printf("system event!\n");
      break;
#if 0
    case SDL_VIDEORESIZE:
      printf("Resizing window to %dx%d\n", event->resize.w, event->resize.h);
      screen = resize_window(event->resize.w, event->resize.h);
      if (! screen) return 1;
      break;
#endif
    case SDL_VIDEOEXPOSE:
      visible = 1;
      break;
#if 1
    case SDL_ACTIVEEVENT:
      if (event->active.state == SDL_APPACTIVE) visible = event->active.gain;
      break;
#endif
  }

  return ret;
}

int handle_key_press(SDL_KeyboardEvent *key) {
  int ret = 0;

  switch (key->keysym.sym) {
    case GUI_PAUSE_KEY: paused = ! paused; break;
    case GUI_FORWARD_KEY: zpos += KEYBOARDSPEED; break;
    case GUI_BACKWARD_KEY: zpos -= KEYBOARDSPEED; break;
    case GUI_LEFT_KEY: xpos -= KEYBOARDSPEED; break;
    case GUI_RIGHT_KEY: xpos += KEYBOARDSPEED; break;
    case GUI_UP_KEY: ypos += KEYBOARDSPEED; break;
    case GUI_DOWN_KEY: ypos -= KEYBOARDSPEED; break;
    case GUI_NEXT_FRAME_KEY: gui_next_frame(); break;
    case GUI_PREV_FRAME_KEY: gui_prev_frame(); break;
    /*
    case SDLK_ESCAPE:
      ret |= 1;
      break;
    */
    case MIDDLE_BUTTON_EMULATION_KEY:
      axishack = 1;
      break;
#if 0
    default:
      printf("key was %u\n", key->keysym.sym);
#endif
  }

  return ret;
}

int handle_key_release(SDL_KeyboardEvent *key) {
  int ret = 0;

  switch (key->keysym.sym) {
    case GUI_LOAD_MESH_KEY: gui_load_mesh(); break;
    case GUI_LOAD_ANIMATION_KEY: gui_load_animation(); break;
    case GUI_LOAD_BACKGROUND_KEY: gui_load_background(); break;
    case GUI_WIREFRAME_KEY: gui_toggle_wireframe(); break;
    case GUI_SHADED_KEY: gui_toggle_shading(); break;
    case GUI_TEXTURED_KEY: gui_toggle_texturing(); break;
    case GUI_BACKGROUND_KEY: gui_toggle_background(); break;
    case GUI_AA_KEY: gui_toggle_aa(); break;
    case GUI_NORMALS_KEY: gui_toggle_normals(); break;
    case GUI_RELOAD_TEXTURES_KEY:
      gui_reload_textures(character);
      gui_reload_textures(weapon);
      break;
#ifdef _MSC_VER
    case GUI_TEXTURES_KEY: gui_spawn_textures_window(0); break;
#endif
    case MIDDLE_BUTTON_EMULATION_KEY:
      axishack = 0;
      break;
  }

  return ret;
}

Uint32 advance_frame(Uint32 interval) {
  if (! paused) gui_next_frame();
  return interval;
}

void process_args(int argc, char **argv) {
  for (int i = 1; i < argc; i++) {
    std::string arg = argv[i];
    if (arg == "--nojoystick") joystick_enabled = 0;
    if (arg == "--notextures") {
      textures_enabled = 0;
      material_hack = 0;
    }
    if (arg == "--bones") show_bones = 1;
    if (arg == "--nomesh") show_mesh = 0;
  }
}
